@@ -86,7 +86,7 @@ class MIDITokenizer:
         # MIDI (being parsed) so that methods processing tracks can access them
         self.current_midi_metadata = {}  # needs to be updated each time a MIDI is read
 
-    def midi_to_tokens(self, midi: MidiFile) -> List[List[Union[int, List[int]]]]:
+    def midi_to_tokens(self, midi: MidiFile, *args, **kwargs) -> List[List[Union[int, List[int]]]]:
         """ Converts a MIDI file in a tokens representation.
         NOTE: if you override this method, be sure to keep the first lines in your method
 
@@ -159,17 +159,28 @@ class MIDITokenizer:
         """
         return [self.vocab.event_to_token[str(event)] for event in events]
 
-    def tokens_to_events(self, tokens: List[int]) -> List[Event]:
+    def tokens_to_events(self, tokens: List[Union[int, List[int]]], multi_voc: bool = False) \
+            -> List[Union[Event, List[Event]]]:
         """ Convert a sequence of tokens in their respective event objects
         You can override this method if necessary
 
         :param tokens: sequence of tokens to convert
+        :param multi_voc: set True if the tokenizer has several vocabularies, e.g. CP Word or Octuple
+                            (default: False)
         :return: the sequence of corresponding events
         """
         events = []
-        for token in tokens:
-            name, val = self.vocab.token_to_event[token].split('_')
-            events.append(Event(name, None, val, None))
+        if multi_voc:
+            for multi_token in tokens:
+                multi_event = []
+                for i, token in enumerate(multi_token):
+                    name, val = self.vocab[i].token_to_event[token].split('_')
+                    multi_event.append(Event(name, None, val, None))
+                events.append(multi_event)
+        else:
+            for token in tokens:
+                name, val = self.vocab.token_to_event[token].split('_')
+                events.append(Event(name, None, val, None))
         return events
 
     def tokens_to_midi(self, tokens: List[List[Union[int, List[int]]]],
@@ -309,7 +320,7 @@ class MIDITokenizer:
         seq.insert(0, self.vocab['SOS_None'])
         seq.append(self.vocab['EOS_None'])
 
-    def _create_vocabulary(self, *args, **kwargs) -> Vocabulary:
+    def _create_vocabulary(self, *args, **kwargs) -> Union[Vocabulary, List[Vocabulary]]:
         """ Creates the Vocabulary object of the tokenizer.
         See the docstring of the Vocabulary class for more details about how to use it.
         NOTE: token index 0 is often used as a padding index during training
@@ -597,6 +608,22 @@ class MIDITokenizer:
         if '_mask' not in params:
             self._mask = False
 
+    def __call__(self, midi: MidiFile, *args, **kwargs):
+        return self.midi_to_tokens(midi, *args, **kwargs)
+
+    def __len__(self):
+        return [len(v) for v in self.vocab] if isinstance(self.vocab, list) else len(self.vocab)
+
+    def __getitem__(self, item: Union[int, str, Tuple[int, int]]) -> Union[str, int]:
+        if isinstance(item, str):
+            return self.vocab.event_to_token[item]
+        elif isinstance(item, int):
+            return self.vocab.token_to_event[item]
+        elif isinstance(item, tuple) and isinstance(self.vocab, list):
+            return self.vocab[item[0]].token_to_event[item[1]]
+        else:
+            raise IndexError('The index must be an integer or a string')
+
 
 def get_midi_programs(midi: MidiFile) -> List[Tuple[int, bool]]:
     """ Returns the list of programs of the tracks of a MIDI, deeping the
